//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/View/Toplevel/SessionModelView.cpp
//! @brief     Implements class SessionModelView
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2018
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#include "GUI/View/Toplevel/SessionModelView.h"
#include "GUI/Model/Data/RealDataModel.h"
#include "GUI/Model/Instrument/InstrumentModel.h"
#include "GUI/Model/Job/JobModel.h"
#include "GUI/Model/Material/MaterialModel.h"
#include "GUI/Model/Sample/SampleModel.h"
#include "GUI/Model/Session/SessionDecorationModel.h"
#include "GUI/View/Main/ProjectManager.h"
#include "GUI/View/PropertyEditor/SessionModelDelegate.h"
#include "GUI/View/Tool/StyleUtils.h"
#include <QToolBar>
#include <QToolButton>
#include <QTreeView>
#include <QVBoxLayout>

SessionModelView::SessionModelView(QWidget* parent, ProjectDocument* document) : QWidget(parent)
{
    auto* layout = new QVBoxLayout(this);
    layout->setMargin(0);

    auto* tabs = new QTabWidget(this);
    auto* m_expandButton = new QToolButton(this);
    m_expandButton->setText("Expand tree");
    m_expandButton->setIcon(QIcon(":/images/toolbar_expand_collapse_tree.svg"));
    m_expandButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    connect(m_expandButton, &QToolButton::clicked, [=]() {
        auto* tree = static_cast<QTreeView*>(tabs->widget(tabs->currentIndex()));
        tree->expandAll();
        tree->resizeColumnToContents(0);
        tree->resizeColumnToContents(1);
    });

    auto* m_collapseButton = new QToolButton(this);
    m_collapseButton->setText("Collapse tree");
    m_collapseButton->setIcon(QIcon(":/images/toolbar_expand_collapse_tree.svg"));
    m_collapseButton->setToolButtonStyle(Qt::ToolButtonTextBesideIcon);
    connect(m_collapseButton, &QToolButton::clicked, [=]() {
        auto* tree = static_cast<QTreeView*>(tabs->widget(tabs->currentIndex()));
        tree->collapseAll();
    });

    auto* toolBar = new QToolBar;
    toolBar->addWidget(m_expandButton);
    toolBar->addWidget(m_collapseButton);
    layout->addWidget(toolBar);
    layout->addWidget(tabs);

    QList<SessionModel*> models{document->instrumentModel(), document->sampleModel(),
                                document->realDataModel(), document->materialModel(),
                                document->jobModel()};

    for (auto* model : models) {
        auto* treeView = new QTreeView(this);
        GUI::Util::Style::setPropertyStyle(treeView);
        treeView->setItemDelegate(new SessionModelDelegate(this));
        treeView->setModel(new SessionDecorationModel(treeView, model));
        tabs->addTab(treeView, model->getModelTag());
    }
}
